#include "mpi.h"

#define MAX_COUNT 1000

int main( argc, argv )
int argc;
char **argv;
{
  MPI_File     fh;
  MPI_Info     info;
  MPI_Status   status;
  MPI_Datatype filetype;
  char         *filename;
  int          buffer[MAX_COUNT];
  int          count, rank;

  MPI_Init( &argc, &argv );
  
  MPI_Info_create( &info );
  /* MPI_INFO_NULL may be used instead of this info */
  MPI_Info_set( info, "collective_buffering", "true" );

  /* The filename is implementation-specific (just as it is
     for all environments).  This assumes that a simple file name will 
     work.  The file name may be over-ridden from the command line */
  filename = "sample.input";
  if (argc > 1 && argv[1]) filename = argv[1];

  /* All processes open the file.  Complex headers (with fixed-sized
     fields) can be read using an MPI datatype */
  MPI_File_open( MPI_COMM_WORLD, filename, 
		 MPI_MODE_RDONLY | MPI_MODE_UNIQUE_OPEN,
		 info, &fh );

  /* Give each process a different part of the data */
  /* Construct a datatype that assigns to processor rank the integers
     from rank*size to (rank+1)*size-1 */
  MPI_Comm_rank( MPI_COMM_WORLD, &rank );
  blocklen = MAX_COUNT;
  index = rank * MAX_COUNT;
  MPI_Type_indexed( 1, &blocklen, &index, MPI_INT, &filetype );
  MPI_Type_commit( &filetype );
  MPI_File_set_view( fh, MPI_DISPLACEMENT_CURRENT, MPI_INT, filetype, 
		     "native", MPI_INFO_NULL );

  MPI_File_read_all( fh, buffer, MAX_COUNT, MPI_INT, &status );

  /* All processes now have all of the data */
  /* Find out how much data was read */
  MPI_Get_count( &status, MPI_INT, &count );
  if (count != MAX_COUNT) {
    fprintf( stderr, "Read %d ints; expected %d\n", count, MAX_COUNT );
  }

  /* The sample file is created by a single process containing consequtive 
     values.  The following code checks that we've got what we expected */
  for (i=0; i<MAX_COUNT; i++) {
    if (buffer[i] != rank * MAX_COUNT + i) {
      fprintf( stderr, "Read %d but expected %d\n", buffer[i], 
	       rank * MAX_COUNT + i );
    }
  }

  if (rank == 0) {
    printf( "Read %d integers on each processor\n", count );
  }

  MPI_File_close( &fh );
  MPI_Type_free( &filetype );

  MPI_Finalize( );
  return 0;
}
