#include "mpi.h"
#include <stdio.h>

#define MAX_COUNT 1000
#define MAX_PROCS 64

/* 
   This writes a single file from all processors containing 
   integers.  It can be used to create a file that the read examples
   can read and check that they've read it correctly.

   This uses the shared file pointers and write ordered to generate
   a file in rank order.
 */

int main( argc, argv )
int argc;
char **argv;
{
  MPI_File   fh;
  MPI_Status status;
  MPI_Info   info;
  char       *filename;
  int        buffer[MAX_COUNT];
  int        i, rank, size;

  MPI_Init( &argc, &argv );
  
  MPI_Comm_rank( MPI_COMM_WORLD, &rank );

  MPI_Info_create( &info );
  /* MPI_INFO_NULL may be used instead of this info */
  MPI_Info_set( info, "collective_buffering", "true" );
  /* file_perm is implementation dependent.  On Unix systems, we hope that
     it will accept the same values as the Unix open (at least!) */
  MPI_Info_set( info, "file_perm", "0666" );
  MPI_Info_set( info, "access_style", "write_once" );

  /* The filename is implementation-specific (just as it is
     for all environments).  This assumes that a simple file name will 
     work.  The file name may be over-ridden from the command line */
  filename = "sample.input";
  if (argc > 1 && argv[1]) filename = argv[1];

  MPI_File_open( MPI_COMM_WORLD, filename, 
		 MPI_MODE_WRONLY | MPI_MODE_UNIQUE_OPEN | MPI_MODE_CREATE,
		 info, &fh );
  MPI_Comm_size( MPI_COMM_WORLD, &size );
  MPI_File_preallocate( fh, MAX_COUNT * size * sizeof(int) );

  for (i=0; i<MAX_COUNT; i++) {
    buffer[i] = rank * MAX_COUNT + i;
  }
  MPI_File_write_ordered( fd, buffer, MAX_COUNT, MPI_INT, &status );
  /* Check status for errors .... */

  printf( "Wrote file\n" );
  MPI_File_close( &fh );
  MPI_Finalize( );
  return 0;
}
