#include "mpi.h"
#include <stdio.h>

#define MAX_COUNT 1000
#define MAX_PROCS 64

/* 
   This writes a single file from a single processor containing 
   integers.  It can be used to create a file that the read examples
   can read and check that they've read it correctly.

   One reason for using a single process to write out is to emphasize that 
   the files are the same, independent of the number of processors used
   to read or write.
 */

int main( argc, argv )
int argc;
char **argv;
{
  MPI_File   fh;
  MPI_Status status;
  MPI_Info   info;
  char       *filename;
  int        buffer[MAX_COUNT];
  int        i, j, rank;

  MPI_Init( &argc, &argv );
  
  MPI_Comm_rank( MPI_COMM_WORLD, &rank );
  /* Only rank 0 does anything */
  if (rank != 0) {
    MPI_Finalize( );
    return 0;
  }

  MPI_Info_create( &info );
  /* MPI_INFO_NULL may be used instead of this info */
  /* file_perm is implementation dependent.  On Unix systems, we hope that
     it will accept the same values as the Unix open (at least!) */
  MPI_Info_set( info, "file_perm", "0666" );
  MPI_Info_set( info, "access_style", "write_once" );

  /* The filename is implementation-specific (just as it is
     for all environments).  This assumes that a simple file name will 
     work.  The file name may be over-ridden from the command line */
  filename = "sample.input";
  if (argc > 1 && argv[1]) filename = argv[1];

  MPI_File_open( MPI_COMM_SELF, filename, 
		 MPI_MODE_WRONLY | MPI_MODE_UNIQUE_OPEN | MPI_MODE_CREATE,
		 info, &fh );
  MPI_File_preallocate( fh, MAX_COUNT * MAX_PROCS * sizeof(int) );
  for (j=0; j<MAX_PROCS; j++) {
    for (i=0; i<MAX_COUNT; i++) {
      buffer[i] = j * MAX_COUNT + i;
    }
    MPI_File_write( fd, buffer, MAX_COUNT, MPI_INT, &status );
    /* Check status for errors .... */
  }
  printf( "Wrote file\n" );
  MPI_File_close( &fh );
  MPI_Finalize( );
  return 0;
}
