C This example handles a 12 x 12 mesh, on 4 processors only.
      program main
      include 'mpif.h'
C       In MPI-2, you may be able to use an MPI module that will give
C       enhanced compile-time error checking
      integer maxn
      parameter (maxn=12)
C
      integer rank, value, size, errcnt, toterr, i, j, itcnt, ierr
      integer i_first, i_last
      integer status(MPI_STATUS_SIZE)
      double precision diffnorm, gdiffnorm
      double precision xlocal(12,12/4+2)
      double precision xnew(12,(12/3)+2)
      double precision x(maxn,maxn)
C
      call MPI_Init( ierr )

      call MPI_Comm_rank( MPI_COMM_WORLD, rank, ierr )
      call MPI_Comm_size( MPI_COMM_WORLD, size, ierr )

      if (size .ne. 4) call MPI_Abort( MPI_COMM_WORLD, 1, ierr )

C xlocal(1,:) is lower ghostpoints, xlocal(maxn+2,:) is upper

C Note that top and bottom processes have one less row of interior
C points 
      i_first = 1
      i_last  = maxn/size
      if (rank .eq. 0)        i_first = i_first + 1
      if (rank .eq. size - 1) i_last  = i_last - 1

C Fill the data as specified
      do i=1,maxn/size
	do j=1,maxn
           xlocal(j,i) = rank
        enddo
      enddo
      do j=1, maxn
	xlocal(j,i_first-1) = -1
	xlocal(j,i_last+1)  = -1
      enddo

      itcnt = 0
 10   continue
C	Send up unless I'm at the top, then receive from below
C	Note the use of xlocal[i] for xlocal[i][0]
	if (rank .lt. size - 1) then
	    call MPI_Send( xlocal[maxn/size], maxn, 
     *                     MPI_DOUBLE_PRECISION, rank + 1, 0, 
     *                     MPI_COMM_WORLD, ierr )
        endif
	if (rank .gt. 0) then
	    call MPI_Recv( xlocal[0], maxn, MPI_DOUBLE_PRECISION, 
     *                rank - 1, 0, MPI_COMM_WORLD, status, ierr )
        endif
C       Send down unless I'm at the bottom
	if (rank .gt. 0) then
	    call MPI_Send( xlocal[1], maxn, MPI_DOUBLE, rank - 1, 1, 
		      MPI_COMM_WORLD )
endif
	if (rank < size - 1) then
	    call MPI_Recv( xlocal[maxn/size+1], maxn, MPI_DOUBLE, rank + 1, 1, 
		      MPI_COMM_WORLD, status )
	endif
C Compute new values (but not on boundary)
	itcnt = itcnt + 1
	diffnorm = 0.0
        do i=i_first,i_last
           do j=1,maxn-2
              xnew[i][j] = (xlocal[i][j+1] + xlocal[i][j-1] +
              xlocal[i+1][j] + xlocal[i-1][j]) / 4.0
              diffnorm += (xnew[i][j] - xlocal[i][j]) * 
              (xnew[i][j] - xlocal[i][j])
           enddo
        enddo
C       Only transfer the interior points
	do i=i_first,i_last
	    do j=1,maxn-2
               xlocal(j,i) = xnew(j,i)
            enddo
         enddo
         call MPI_Allreduce( diffnorm, gdiffnorm, 1, 
     *                       MPI_DOUBLE_PRECISION, MPI_SUM,
     * 	                     MPI_COMM_WORLD, ierr )
         gdiffnorm = sqrt( gdiffnorm )
         if (rank .eq. 0) then
            print *, 'At iteration ', itcnt, ', diff is ', gdiffnorm
         endif
         if (gdiffnorm .gt. 1.0e-2 .and. itcnt .lt. 100) goto 10

C Collect the data into x and print it
C This code (and the declaration of x above) are the only changes to the
C Jacobi code
        call MPI_Gather( xlocal[1], maxn * (maxn/size), 
     *                   MPI_DOUBLE_PRECISION,
     *                   x, maxn * (maxn/size), MPI_DOUBLE_PRECISION, 
     *            	 0, MPI_COMM_WORLD, ierr )
        if (rank .eq. 0) then
           print *, 'Final solution is'
           for (i=maxn-1 i>=0 i--) {
           for (j=0 j<maxn j++) 
		printf( "%f ", x[i][j] )
	    printf( "\n" )
	}
    }

      call MPI_Finalize( ierr )
      end
